from typing import List, Dict, Any
from csle_common.dao.emulation_config.node_flags_config import NodeFlagsConfig
from csle_common.dao.emulation_config.flag import Flag
from csle_base.json_serializable import JSONSerializable


class FlagsConfig(JSONSerializable):
    """
    A DTO representing the set of flags in an emulation environment
    """

    def __init__(self, node_flag_configs: List[NodeFlagsConfig]):
        """
        Initializes the DTO

        :param node_flag_configs: the list of flags
        """
        self.node_flag_configs = node_flag_configs

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "FlagsConfig":
        """
        Converts a dict representation to a an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = FlagsConfig(
            node_flag_configs=list(map(lambda x: NodeFlagsConfig.from_dict(x), d["node_flag_configs"]))
        )
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation
        
        :return: a dict representation of the object
        """
        d = {}
        d["node_flag_configs"] = list(map(lambda x: x.to_dict(), self.node_flag_configs))
        return d

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return ",".join(list(map(lambda x: str(x), self.node_flag_configs)))

    def get_flags_for_ips(self, ips: List[str]) -> List[Flag]:
        """
        Get all flags for a list of ip addresses

        :param ips: the list of ip addresses to get flags for
        :return: the list of flags
        """
        flags: List[Flag] = []
        for node_flag_config in self.node_flag_configs:
            if node_flag_config.ip in ips:
                flags = flags + node_flag_config.flags
        return flags

    @staticmethod
    def from_json_file(json_file_path: str) -> "FlagsConfig":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return FlagsConfig.from_dict(json.loads(json_str))

    def copy(self) -> "FlagsConfig":
        """
        :return: a copy of the DTO
        """
        return FlagsConfig.from_dict(self.to_dict())

    def create_execution_config(self, ip_first_octet: int) -> "FlagsConfig":
        """
        Creates a new config for an execution

        :param ip_first_octet: the first octet of the IP of the new execution
        :return: the new config
        """
        config = self.copy()
        config.node_flag_configs = list(map(lambda x: x.create_execution_config(ip_first_octet=ip_first_octet),
                                        config.node_flag_configs))
        return config



import subprocess
from unittest.mock import patch, MagicMock
from csle_common.controllers.installation_controller import InstallationController


class TestInstallationControllerSuite:
    """
    Test suite for installation_controller
    """

    @patch("subprocess.Popen")
    def test_install_all_emulations(self, mock_popen) -> None:
        """
        Test the method that installs all emulations in the metastore

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.install_all_emulations()
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/emulation-system/envs/ && make install", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_install_emulation(self, mock_popen) -> None:
        """
        Test the method that installs a given emulation in the metastore

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.install_emulation(emulation_name="name")
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/emulation-system/envs/ && make install_name", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_uninstall_emulation(self, mock_popen) -> None:
        """
        Test the method that uninstalls a given emulation in the metastore

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.uninstall_emulation(emulation_name="name")
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/emulation-system/envs/ && make uninstall_name", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_uninstall_emulations(self, mock_popen) -> None:
        """
        Test the method that uninstalls all emulations in the metastore

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.uninstall_all_emulations()
        mock_popen.assert_called_once_with("cd $CSLE_HOME/emulation-system/envs/ && make uninstall",
                                           stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_install_all_simulations(self, mock_popen) -> None:
        """
        Test the method that installs all simulations in the metastore

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.install_all_simulations()
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/simulation-system/envs/ && make install", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_install_simulation(self, mock_popen) -> None:
        """
        Test the method that installs a given simulation in the metastore

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.install_simulation(simulation_name="name")
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/simulation-system/envs/ && make install_name",
            stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_uninstall_simulation(self, mock_popen) -> None:
        """
        Test the method that uninstalls a given simulation in the metastore

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.uninstall_simulation(simulation_name="name")
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/simulation-system/envs/ && make uninstall_name", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_uninstall_all_simulations(self, mock_popen) -> None:
        """
        Test the method that uninstalls all simulations in the metastore

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.uninstall_all_simulations()
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/simulation-system/envs/ && make uninstall", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_install_derived_images(self, mock_popen) -> None:
        """
        Test the method that installs all derived Docker images

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.install_derived_images()
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/emulation-system/derived_images/ && make build", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_uninstall_derived_images(self, mock_popen) -> None:
        """
        Test the method that uninstalls all derived Docker images

        :param mock_popen: mock Popen method

        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.uninstall_derived_images()
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/emulation-system/derived_images/ && make rm_image", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_uninstall_derived_image(self, mock_popen) -> None:
        """
        Test the method that uninstalls a given derived Docker images

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.uninstall_derived_image(image_name="image")
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/emulation-system/derived_images/ && make rm_image", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_install_base_images(self, mock_popen) -> None:
        """
        Test the method that installs all base Docker images

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.install_base_images()
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/emulation-system/base_images/ && make build", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_install_base_image(self, mock_popen) -> None:
        """
        Test the method that installs a given base Docker images

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.install_base_image(image_name="name")
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/emulation-system/base_images/ && make name", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_uninstall_base_images(self, mock_popen) -> None:
        """
        Test the method that uninstalls all base Docker images

        :param mock_popen: mock Popen method

        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.uninstall_base_images()
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/emulation-system/base_images/ && make rm_image", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_uninstall_base_image(self, mock_popen) -> None:
        """
        Test the method that uninstalls a given base Docker images

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.uninstall_base_image(image_name="name")
        mock_popen.assert_called_once_with(
            "cd $CSLE_HOME/emulation-system/base_images/ && make rm_name", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_uninstall_metastore(self, mock_popen) -> None:
        """
        Test the method that uninstalls the metastore

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.uninstall_metastore()
        mock_popen.assert_called_once_with("cd $CSLE_HOME/metastore/ && make clean", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4

    @patch("subprocess.Popen")
    def test_install_metastore(self, mock_popen) -> None:
        """
        Test the method that installs the metastore

        :param mock_popen: mock Popen method
        :return: None
        """
        # mock the process returned by 'subprocess.Popen', which includes 'stdout' and 'poll'
        mock_process = MagicMock()
        mock_process.stdout.read.side_effect = [b"c", b"d", b"", b"$"]
        mock_process.poll.side_effect = [None, None, None, 0]  # return 0, means stop running
        mock_popen.return_value = mock_process
        InstallationController.install_metastore()
        mock_popen.assert_called_once_with("cd $CSLE_HOME/metastore/ && make build", stdout=subprocess.PIPE, shell=True)
        assert mock_process.stdout.read.call_count == 4
        assert mock_process.poll.call_count == 4



from typing import List, Dict, Any
from csle_common.dao.system_identification.mcmc_posterior import MCMCPosterior
from csle_common.dao.system_identification.system_model import SystemModel
from csle_common.dao.system_identification.system_model_type import SystemModelType


class MCMCSystemModel(SystemModel):
    """
    A system model (list of posterior distributions) made up of posterior distributions
    """

    def __init__(self, emulation_env_name: str, emulation_statistic_id: int,
                 posteriors: List[MCMCPosterior], descr: str):
        """
        Initializes the object

        :param emulation: the emulation that this system model is for
        :param emulation_statistic_id: the emulation statistic that this model was built from
        :param posteriors: the list of posteriors
        :param descr: description of the model
        """
        super(MCMCSystemModel, self).__init__(descr=descr, model_type=SystemModelType.MCMC)
        self.posteriors = posteriors
        self.emulation_env_name = emulation_env_name
        self.emulation_statistic_id = emulation_statistic_id
        self.id = -1

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "MCMCSystemModel":
        """
        Converts a dict representation of the DTO into an instance

        :param d: the dict to convert
        :return: the converted instance
        """
        dto = MCMCSystemModel(
            posteriors=list(map(lambda y: MCMCPosterior.from_dict(y), d["posteriors"])),
            emulation_env_name=d["emulation_env_name"], emulation_statistic_id=d["emulation_statistic_id"],
            descr=d["descr"]
        )
        if "id" in d:
            dto.id = d["id"]
        return dto

    def to_dict(self) -> Dict[str, Any]:
        """
        :return: a dict representation of the DTO
        """
        d: Dict[str, Any] = {}
        d["posteriors"] = list(map(lambda y: y.to_dict(), self.posteriors))
        d["emulation_env_name"] = self.emulation_env_name
        d["emulation_statistic_id"] = self.emulation_statistic_id
        d["descr"] = self.descr
        d["id"] = self.id
        d["model_type"] = self.model_type
        return d

    def __str__(self):
        """
        :return: a string representation of the DTO
        """
        return f"posteriors: {self.posteriors}, " \
               f"emulation_env_name: {self.emulation_env_name}, " \
               f"emulation_statistic_id: {self.emulation_statistic_id}," \
               f"descr: {self.descr}, model_type: {self.model_type}"

    def copy(self) -> "MCMCSystemModel":
        """
        :return: a copy of the DTO
        """
        return self.from_dict(self.to_dict())

    @staticmethod
    def from_json_file(json_file_path: str) -> "MCMCSystemModel":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return MCMCSystemModel.from_dict(json.loads(json_str))
